#!/bin/sh
# ============================================================================
#
# This file is part of the 'MetaRepository for zypkgs' package
# 
# This program is free software; you can redistribute it and/or modify it 
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Author: Mijzelf <Mijzelf@live.com>
#
# ============================================================================
#

ENABLE_MULTITHREADING=1

#############################################
## Sets the paths of files in ZYPKG_INFO.tgz of the terget NAS
SetGlobalPaths()
{
	local targetfw=$1
	
	if [ ${targetfw} -eq 5 ] ; then
		ZYPKGS=/i-data/.system/zy-pkgs/ZYPKGS
		DEPS=/i-data/.system/zy-pkgs/ZYPKG_DEPS
		LANGLIST=/i-data/.system/zy-pkgs/pkggui/ZYPKG_LANG_LIST
		SUPPKG=/i-data/.system/zy-pkgs/SUPPKG
	else
		ZYPKGS=/i-data/md0/admin/zy-pkgs/ZYPKGS
		DEPS=/usr/local/zy-pkgs/etc/init.d/ZYPKG_DEPS
		LANGLIST=/usr/local/zy-pkgs/gui/ZYPKG_LANG_LIST
	fi
}

RenamePackageFile()
{
	[ ! -f $1 ] && return
#	echo -n "RenamePackageFile $@ " >&2
	
	local package=` grep ^Package $1 | cut -d ' ' -f 2 `
	local version=` grep ^Version $1 | cut -d ' ' -f 2 ` 
	version=` echo $version | sed 's/pkg/ /' | cut -d ' ' -f 2 `
	local filename=` grep ^Filename $1 | cut -d ' ' -f 2 `
	
	[ -z $filename ] && rm $1 && echo empty >&2 && return
	
	local dir=` dirname $1 `
#	echo ${dir}/${filename} >&2
	mv $1 ${dir}/${filename}
	echo ${package} ${2}-${version}-${3} ` basename ${dir} `/${filename} >>${dir}.list
}

FetchAll()
{
	local target=$1
	shift
	local url=$1
	shift
	for file in $@
	do
		Fetch ${url}${file} ${target}
		[ $? -eq 0 ] && echo ${file} && return
	done
}

FetchZYPKG_INFO()
{
	echo FetchZYPKG_INFO $@ >&2

	url=$1
	postfix=$2
	
	filename=${SCRATCH}/` URLtoFilename $url `
	echo $url >${filename}.url
	
	[ -z $MD5SUM ] && MD5SUM=` FindApp md5sum `
	
	local filelist="ZYPKG_INFO.tgz ZYPKGS"
	[ "$MEDION" = "1" ] && filelist="$filelist PKG_INFO.tgz PKGS"
	
	local result=` FetchAll ${filename}.tmp $url $filelist `
	local md5="bogus"
	local prefix="1"
	
	case $result in
		ZYPKG_INFO.tgz)
			md5=` $MD5SUM ${filename}.tmp | cut -d ' ' -f 1 `
			[ -f ${filename}.md5 ] && [ "$md5" = "` cat ${filename}.md5 `" ] && rm ${filename}.tmp && return 1 # Nothin changed
			
			mv ${filename}.tmp ${filename}.tgz
			rm -rf ${filename}/
			mkdir -p ${filename}
		
			tar xf ${filename}.tgz -C ${filename}/
			if [ $? -ne 0 ] ; then
				rm ${filename}.tgz
				rm -rf ${filename}/
				rm ${filename}.md5
				return 0
			fi
			rm ${filename}.tgz
			;;
		PKG_INFO.tgz)
			md5=` $MD5SUM ${filename}.tmp | cut -d ' ' -f 1 `
			[ -f ${filename}.md5 ] && [ "$md5" = "` cat ${filename}.md5 `" ] && rm ${filename}.tmp && return 1 # Nothin changed
			
			mv ${filename}.tmp ${filename}.tgz
			rm -rf ${filename}/
			mkdir -p ${filename}
		
			tar xf ${filename}.tgz -C ${filename}/
			if [ $? -ne 0 ] ; then
				rm ${filename}.tgz
				rm -rf ${filename}/
				rm ${filename}.md5
				return 0
			else
				mv ${filename}/i-data/md0/admin/package/PKGS ${filename}/i-data/md0/admin/package/ZYPKGS
				mv ${filename}/i-data/md0/admin/package/ ${filename}/i-data/md0/admin/zy-pkgs/
			fi
			prefix="2"
			rm ${filename}.tgz
			;;
		ZYPKGS)
			mkdir -p ` dirname ${filename}${ZYPKGS} `
			mv ${filename}.tmp ${filename}${ZYPKGS}
			md5=` $MD5SUM ${filename}${ZYPKGS} | cut -d ' ' -f 1 `
		
			[ -f ${filename}.md5 ] && [ "$md5" = "` cat ${filename}.md5 `" ] && return 1 # Nothing changed
			;;
		PKGS)
			mkdir -p ` dirname ${filename}${ZYPKGS} `
			mv ${filename}.tmp ${filename}${ZYPKGS}
			md5=` $MD5SUM ${filename}${ZYPKGS} | cut -d ' ' -f 1 `
		
			[ -f ${filename}.md5 ] && [ "$md5" = "` cat ${filename}.md5 `" ] && return 1 # Nothing changed
			prefix="2"
			;;
		*)
			echo Failed >&2
			# Not found. If md5 file exists, things have changed
			if [ -f ${filename}.md5 ] ; then
				rm ${filename}.md5 
				[ -f ${filename}.list ] && rm ${filename}.list
				return 0
			fi
			return 1
	esac
	
	echo $md5 >${filename}.md5
	[ -f ${filename}.list ] && rm ${filename}.list 
	
	while read line
	do
		line=` StripLeadingAndTrailingSpaces "$line" `
		if [ "$line" = "" ] ; then
			RenamePackageFile ${filename}/tmp ${prefix} ${postfix}
			continue
		fi
		
		echo $line >> ${filename}/tmp
	
	done <${filename}${ZYPKGS}
	
	RenamePackageFile ${filename}/tmp ${prefix} ${postfix}
	return 0
}

HandleRepo()
{
	local postfix=$1
	shift
	local subdir=$1
	shift
	local url=$1
	shift
	local name="$@"
	
	local plus=''
	# Does it end on a '+'?
	if echo $url | grep -e "+$" >/dev/null
	then
		url=` echo $url | sed 's|+$||g' `
		plus=${subdir}
	fi
	
	# Doesn't it end on a /?
	if ! echo $url | grep -e "/$" >/dev/null
	then
		url=${url}/
	fi
	
	url=${url}${plus}
	local filename=` URLtoFilename ${url} `
	[ ! -z "$name" ] && echo "$name" >${SCRATCH}/${filename}.name || rm ${SCRATCH}/${filename}.name

	local tmpdir=/tmp/.MetaRepository/tmp
	mkdir -p ${tmpdir}
	echo ${filename} >>${tmpdir}/repos

	FetchZYPKG_INFO ${url} ${postfix}

	[ $? -eq 0 ] && touch ${tmpdir}/changed
}
		
HandleInstalled()
{
	local dir=${SCRATCH}/Installed
	mkdir -p ${dir}
	rm -rf ${dir}/*
	
	echo Installed >${dir}.name
	
	if [ -d /usr/local/zy-pkgs ] ; then
		echo /i-data/md0/admin/zy-pkgs/ >${dir}.url
		cp /usr/local/zy-pkgs/zypkg_conf/info/*.control ${dir}/
	else
		echo /i-data/.system/zy-pkgs/ >${dir}.url
		grep Installed-Rule /etc/zyxel/pkg_conf/status | awk '{print $2}' | while read line
		do
			cp ${line}zypkg_conf/info/*.control ${dir}/
		done
	fi
	
	[ -f ${dir}.list ] && rm ${dir}.list
	
	for file in ` ls ${dir}/ `
	do
		local package=` grep ^Package ${dir}/${file} | cut -d ' ' -f 2 `
		local version=` grep ^Version ${dir}/${file} | cut -d ' ' -f 2 ` 
		version=` echo $version | sed 's/pkg/ /' | cut -d ' ' -f 2 `
		local filename=` grep ^Filename ${dir}/${file} | cut -d ' ' -f 2 `

		mv ${dir}/${file} ${dir}/${filename}
		echo "${package} 0-${version}-0 Installed/${filename}" >>${dir}.list
	done

	local tmpdir=/tmp/.MetaRepository/tmp
	[ -f ${dir}.list ] && echo Installed >>${tmpdir}/repos
	
	md5=` $MD5SUM ${dir}.list | cut -d ' ' -f 1 `
	[ -f ${dir}.md5 ] && [ "` cat ${dir}.md5 `" = "${md5}" ] && return 1 # Nothing changed

	echo ${md5} >${dir}.md5

	touch ${tmpdir}/changed
	return 0
}

LogFilter()
{
    local thread=$1
    echo $thread start: ` date +%T ` >&2
    
    while read line 
    do
	    echo $thread: $line >&2
    done
    
    echo $thread stop: ` date +%T ` >&2
}

HandleZYPKG_INFO()
{
	echo HandleZYPKG_INFO $@ >&2
	
	local targetfw=4
	
	if echo $1 | grep NAS5 >/dev/null
	then
		targetfw=5
	fi

	if echo $1 | grep NAS3 >/dev/null
	then
		targetfw=5
	fi

	SetGlobalPaths ${targetfw}

	local subdir=""
	for nr in 1 2 3 4 5 6
	do
		local dir=` echo $1 | cut -d '/' -f $nr `
		[ -z ${dir} ] && continue
		subdir="${subdir}${dir}/"
	done
	
	
	local tmpdir=/tmp/.MetaRepository/tmp
	mkdir -p $tmpdir
	for file in changed repos $$.ReadConfig.$$
	do
		[ -f ${tmpdir}/${file} ] && rm ${tmpdir}/${file}
	done
	
	ReadConfig Repositories >${tmpdir}/$$.ReadConfig.$$
	local postfix=9
	
	while read line
	do
		if [ -z "${ENABLE_MULTITHREADING}" -o "${ENABLE_MULTITHREADING}" = "0" ] ; then
			HandleRepo $postfix $subdir $line 
		else
			HandleRepo $postfix $subdir $line 2>&1 | LogFilter $postfix &
		fi
		[ $postfix -gt 0 ] && let postfix=postfix-1

	done <${tmpdir}/$$.ReadConfig.$$
	rm ${tmpdir}/$$.ReadConfig.$$

	# HandleInstalled should only be executed if *this* is the Target NAS. But how to figure that out?
	HandleInstalled

	[ ! -z "${ENABLE_MULTITHREADING}" -a "${ENABLE_MULTITHREADING}" != "0" ] && wait
	
	local changed=0
	[ -f ${tmpdir}/changed ] && rm ${tmpdir}/changed && changed=1
	
	[ $changed -eq 0 ] && [ -f ${SCRATCH}/${subdir}/ZYPKG_INFO.tgz ] && echo Nothing changed >&2 && return
	
	listfiles=""
	for repo in ` cat ${tmpdir}/repos `
	do
		[ -f ${SCRATCH}/${repo}.list ] && listfiles="$listfiles ${SCRATCH}/${repo}.list" 
		[ -f ${SCRATCH}/${repo}.depscnt ] && rm ${SCRATCH}/${repo}.depscnt
	done
	
	[ "$listfiles" = "" ] && return
	
	mkdir -p ${SCRATCH}/${subdir}
	local listfile=${SCRATCH}/${subdir}List
	cat $listfiles >${listfile}.$$
	
	# Apply pushlist
	echo Pushlist... >&2
	ReadConfig Pushlist >${tmpdir}/$$.ReadConfig.$$
	while read line
	do
		local search=` echo "$line " | cut -d ' ' -f 1 `
		local second=` echo "$line " | cut -d ' ' -f 2- `
		local replace="${search} z"
		[ ! -z ${second} ] && search="${search} ${second}" && replace="${replace}${second}"
		/bin/sed -i "s/^${search}/${replace}/I" ${listfile}.$$
	done <${tmpdir}/$$.ReadConfig.$$
	
	# Apply blacklist
	echo Blacklist... >&2
	ReadConfig Blacklist >${tmpdir}/$$.ReadConfig.$$
	while read line
	do
		local pattern=` echo "$line " | cut -d ' ' -f 1 `
		local second=` echo "$line " | cut -d ' ' -f 2- `
		[ ! -z ${second} ] && pattern="${pattern} ${second}"
		/bin/sed -i "s/^${pattern}/#${pattern}/I" ${listfile}.$$
		echo /bin/sed -i "s/^${pattern}/#${pattern}/I" ${listfile}.$$ >&2
	done <${tmpdir}/$$.ReadConfig.$$
	rm ${tmpdir}/$$.ReadConfig.$$
	
	# Sort it
	echo Sort... >&2
	cat ${listfile}.$$ | sort -r -f >${listfile}
	rm ${listfile}.$$
	
	# And take one from each offered package
	echo Choose packages... >&2
	[ ! -z ${ZYPKGS} ] && local targetZYPKGS=${SCRATCH}/${subdir}.${ZYPKGS}
	[ ! -z ${LANGLIST} ] && local targetLANGLIST=${SCRATCH}/${subdir}.${LANGLIST}
	[ ! -z ${DEPS} ] && local targetDEPS=${SCRATCH}/${subdir}.${DEPS}
	[ ! -z ${SUPPKG} ] && local targetSUPPKG=${SCRATCH}/${subdir}.${SUPPKG}
	
	for file in $targetZYPKGS $targetLANGLIST $targetDEPS $targetSUPPKG
	do
		mkdir -p ` dirname $file `
		[ -f $file ] && rm $file
	done
	
	local latestpackage="# None "
	while read line
	do
		local package=` echo $line | cut -d ' ' -f 1 `
		case $package in
			${latestpackage})
				continue
				;;
			\#*)
				continue
				;;
		esac
		latestpackage=${package}
		
		local splitfile=` echo $line | cut -d ' ' -f 3 `
		local dir=` dirname $splitfile `
		local name=''
		[ -f ${SCRATCH}/${dir}.name ] && name="<i>(` cat ${SCRATCH}/${dir}.name `)</i> " 

		if [ ! -z ${targetZYPKGS} ] ; then
			cat ${SCRATCH}/${splitfile} | sed "s|^Description:.|Description: ${name}|" >>$targetZYPKGS
			echo >>$targetZYPKGS
		fi
		
		local repo=` dirname ${SCRATCH}/${splitfile} `
		local splitfilename=` basename ${splitfile} `
		
		# Create URL file, to know where to find the package
		echo ` cat ${repo}.url `${splitfilename} >${SCRATCH}/${subdir}${splitfilename}
		if [ ! -z ${targetLANGLIST} ] ; then
			# Add LANGLIST line to target LIST, if applicable
			[ -f ${repo}${LANGLIST} ] && grep -i "^var ${package}=" ${repo}${LANGLIST} | sed "s|':'|':'${name}|g" >>$targetLANGLIST
		fi
		if [ ! -z ${targetDEPS} ] ; then
			# Add DEPS lines, if applicable
			[ -f ${repo}${DEPS} ] && grep -i ^${package} ${repo}${DEPS} >>${repo}.depscnt
		fi
	done <${listfile}
	
	if [ ! -z ${targetDEPS} ] ; then
		# Seach repo which has a DEPS which offers the most 
		# Tha actual order is important, so we can't just randomly create our own
		echo Choose DEPS... >&2
		local maxvalue=0
		for repo in ` cat ${tmpdir}/repos `
		do
			[ ! -f ${SCRATCH}/${repo}.depscnt ] && continue
		
			local value=` wc -l ${SCRATCH}/${repo}.depscnt | cut -d ' ' -f 1 `
			rm ${SCRATCH}/${repo}.depscnt
			if [ -z $maxrepo ] || [ $value -gt $maxvalue ] ; then
				maxrepo=$repo
				maxvalue=$value 
			fi
		done
	
		[ ! -z $maxrepo ] && cp ${SCRATCH}/${maxrepo}/${DEPS} $targetDEPS
	fi
	
	if [ ! -z ${targetSUPPKG} ] ; then
		# Search repo which has a SUPPKG
		echo Choose SUPPKG... >&2
		for repo in ` cat ${tmpdir}/repos `
		do
			if [ -f ${SCRATCH}/${repo}${SUPPKG} ] ; then
				cp ${SCRATCH}/${repo}${SUPPKG} ${SCRATCH}/${subdir}.${SUPPKG}
				break
			fi
		done
	fi
	
	echo Create ZYPKG_INFO.tgz >&2
	local dirs=i-data
	chmod -R 777 ${SCRATCH}/${subdir}/i-data
	[ -d ${SCRATCH}/${subdir}/usr ] && chmod -R 755 ${SCRATCH}/${subdir}/usr && dirs="$dirs usr"

	/tmp/.MetaRepository/bin/tar czf ${SCRATCH}/${subdir}ZYPKG_INFO.tgz --owner=root --group=root -C ${SCRATCH}/${subdir} $dirs
}


HandlePKG_INFO()
{
	echo HandlePKG_INFO $@ >&2

	local subdir=""
	for nr in 1 2 3 4 5 6
	do
		local dir=` echo $1 | cut -d '/' -f $nr `
		[ -z ${dir} ] && continue
		subdir="${subdir}${dir}/"
	done

	local md5="BOGUS"
	local ss=${SCRATCH}/${subdir}
	[ -z $MD5SUM ] && MD5SUM=` FindApp md5sum `
	[ -f ${ss}ZYPKG_INFO.tgz ] && md5=` $MD5SUM ${ss}ZYPKG_INFO.tgz | cut -d ' ' -f 1 `

	MEDION=1
	HandleZYPKG_INFO "$@"

	[ -f ${ss}ZYPKG_INFO.tgz ] && [ "` $MD5SUM ${ss}ZYPKG_INFO.tgz | cut -d ' ' -f 1 `" != "${md5}" ] && [ -f ${ss}PKG_INFO.tgz ] && rm ${ss}PKG_INFO.tgz

	[ -f ${ss}PKG_INFO.tgz ] && return
	[ ! -f ${ss}ZYPKG_INFO.tgz ] && return

	echo Create PKG_INFO.tgz >&2
	local packagedir=${ss}i-data/md0/admin/package
	[ -d $packagedir ] && rm -rf $packagedir
	
	mv ${ss}i-data/md0/admin/zy-pkgs $packagedir
	mv ${packagedir}/ZYPKGS ${packagedir}/PKGS
	
	sed -i 's/zypkg/pkg/' ${packagedir}/PKGS
	sed -i 's/zpkg/pkg/' ${packagedir}/PKGS
	sed -i 's/ure:.arm/ure: arm11/' ${packagedir}/PKGS
	sed -i 's/arm1111/arm11/' ${packagedir}/PKGS
	
	for file in ` ls ${ss}*.zpkg `
	do
		local new=` echo $file | sed 's/zpkg$/pkg/' `
		mv $file $new
	done

	/tmp/.MetaRepository/bin/tar czf ${ss}PKG_INFO.tgz --owner=root --group=root -C ${ss} i-data usr 
}
