#!/bin/sh
# ============================================================================
#
# This file is part of the 'MetaRepository for zypkgs' package
# 
# This program is free software; you can redistribute it and/or modify it 
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# Author: Mijzelf <Mijzelf@live.com>
#
# ============================================================================
#

IndexGet()
{
	HtmlHeader

	local bgimage=""
	local rel=` grep RewriteRule /etc/service_conf/httpd_zld.conf | tail -n 1 | awk '{print $3}' `
	if echo $rel | grep playzone >/dev/null
	then
		bgimage="background-image:url(${rel}images/home_bg.png);"
	fi
	local repos=` ReadConfig Comments Repositories `
echo ==="$repos"==== >&2
	local nrrepos=` echo -e "$repos" | wc -l `
	local blacklist=` ReadConfig Comments Blacklist `
	local nrblacklist=` echo -e "$blacklist" | wc -l `
	local pushlist=` ReadConfig Comments Pushlist `
	local nrpushlist=` echo -e "$pushlist" | wc -l `

	if [ "$NOPASV" = "true" ] ; then
		NOPASV_UI=CHECKED
	fi 

cat >&1 <<__EOF__
<html>
<head>
	<title>Meta Repository</title>
	<style>
	body
	{
		background-color:MidnightBlue;
		color:lemonchiffon;
		${bgimage}
	}
	textarea
	{
		background-color:lemonchiffon;
		color:black;
	}
	input
	{
		background-color:MidnightBlue;
		color:lemonchiffon;
		font-size:140%;
		${bgimage};
	}
	a
	{
		color:Chartreuse;
	}
	</style>
</head>
<body>
<H2>MetaRepository for zypkgs</H2>
<form method="GET" >
<H4>Repositories</H4>Urls of repostories. 
Supported are http, https, ftp and local filesystem. 
A '+' at the end of the url will be exchanged by &lt;NAS&gt;/zypkg/&lt;FW&gt;, which is the default for ZyXEL repositories. 
If you add some extra word after a space after the url, it will show up in the description column in the webinterface.
Lines starting with a '#' are ignored.
<br><textarea name="Repositories" rows=${nrrepos} cols=90 >${repos}</textarea>
<br><input name="nopasv" type="checkbox" $NOPASV_UI >Do not use PASV for ftp
<H4>Blacklist</H4>You can blacklist certain packages/versions.
<br><textarea name="Blacklist" rows=${nrblacklist} cols=90 >${blacklist}</textarea>
<br><br><input type="submit" value="Apply">
</form>
<H4>Release notes</H4>
20151005zypkg012:
<ul><li>Added repo names which show up in the package manager webinterface</li>
<li>Added multithreading. The repos are fetched simultaneously instead of sequnetially</li></ul>
20150916zypkg011:
<ul><li>Added support for the NAS500 series, firmware 5.10 and newer</li>
<li>Removed the 'donate' button</li></ul>
20140529zypkg010:
<ul><li>The firmware really doesn't like it when more instances of httpd are running. When it's running as httpd it's killed sometimes. 
When using symlinks or bindmounts the firmware webserver will not be restarted when needed, because start-stop-daemon detects it's already running. 
Solved it by using inetd, which directly calls the cgi script. Who needs a webserver?</li></ul>
20140520zypkg009:
<ul><li>Changed sort strategy. Instead of sorting on <i>some-version</i>zypkg<i>pkg-version</i>, now is sorted on <i>pkg-version</i>, just like the zypkg-manager does. So the pushlist is no longer necessary.</li>
<li>Added repo priority. When two equal packages are offered by different repo's, it was unpredictable which one was used. Now the sequence in the repo-list is also the priority sequence.</li>
<li>Found what was killing my server. The firmware did sometimes a 'killall httpd'. Solved it by using a symlink</li></ul>
20140418zypkg008:
<ul><li>Fixed a bug which kept a Medion from installing original Medion packages</li>
<li>Limited the number of retries of wget, to protect the script against a non-existing host.</li></ul>
20140407zypkg007:
<ul><li>Fixed <a href="http://www.zyxelforum.de/new-zypkg-metarepository-p44303.html#p44303">this bug</a> by using ftp passive mode</li></ul>
20140405zypkg005:
<ul>
<li>Some internal database files weren't cleaned up in time</li>
<li>Added support for Medion boxes</li>
<li>When an installed package is dropped from the upstream repo, it's no longer maintainable in the packages menu. On a Medion the whole list vanishes. To solve this, I add these orphaned packages to the generated repo file.</li>
</ul>
20140401zypkg004:
<ul>
<li>Fixed <a href="http://forum.nas-central.org/viewtopic.php?p=82079#p82079">this bug</a>, ownership and accessrights of certain directories were borked.</li>
</ul>
20140310zypkg003:
<ul>
<li>Fixed a bug which kept the package from upgrading itself.</li>
</ul>
20140308zypkg002:
<ul>
<li>Some cosmetical updates</li>
<li>The used webserver was killed by *something*. Solved it by wrapping it in a script, and automatically restart it.</li>
</ul>
20140303zypkg001: 
<ul><li>Initial release.</li></ul>
<hr>
<H4><a href="http://zyxel.nas-central.org/wiki/3rd_party_zypkgs#MetaRepository">(External) Wiki entry</a></H4>
</body>
__EOF__

}

IndexPut()
{
	. ${RELDIR}/cgi-bash.sh
	
	cgi_getvars GET ALL
	if [ "on" = "$nopasv" ] ; then
		NOPASV=true
	else
		NOPASV=false
	fi
	
	cat >${CONFIG} <<__EOF2__
SCRATCH=${SCRATCH} 
LISTEN_AT=${LISTEN_AT}
NOPASV=${NOPASV}

# Some trick to keep the rest of the content outside the script:
cat >/dev/null <<__EOF__

[Repositories]
` echo "$Repositories" | tr '\015' '\012' `
[Blacklist]
` echo "$Blacklist" | tr '\015' '\012' `
[Pushlist]
` echo "$Pushlist" | tr '\015' '\012' `
[End]

__EOF__
__EOF2__

rm ${SCRATCH}/*.md5

	HtmlHeader
cat >&1 <<__EOF__
<html>
<head>
    <meta http-equiv="refresh" content="0; url=${SCRIPT_URL}">
</head>
<body></body>
</html>
__EOF__
}


Index_html()
{
	[ "${QUERY_STRING}" = "" ] && IndexGet && exit 0
	IndexPut
	exit 0
}

